@@ -1,7 +1,7 @@
-
 {-# LANGUAGE FlexibleContexts  #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE RecordWildCards   #-}
+{-# LANGUAGE TupleSections     #-}
 {-# LANGUAGE TypeOperators     #-}
 
 -----------------------------------------------------------------------------
@@ -28,6 +28,7 @@ module Text.BlogLiterately.Transform
     , highlightOptsXF
     , passwordXF
     , titleXF
+    , rawtexifyXF
     , wptexifyXF
     , ghciXF
     , uploadImagesXF
@@ -68,6 +69,9 @@ import           Data.Maybe                        (fromMaybe)
 import           Data.Monoid                       (mappend)
 import           Data.Monoid                       (mempty, (<>))
 import qualified Data.Set                          as S
+import           Data.Text                         (Text)
+import qualified Data.Text                         as T
+import qualified Data.Text.Lazy                    as LT
 import           Data.Traversable                  (traverse)
 import           Network.HTTP                      (getRequest, getResponseBody,
                                                     simpleHTTP)
@@ -79,7 +83,7 @@ import           System.IO                         (hFlush, stdout)
 import           Text.Blaze.Html.Renderer.String   (renderHtml)
 import           Text.CSL.Pandoc                   (processCites')
 import           Text.HTML.TagSoup
-import           Text.Pandoc
+import           Text.Pandoc                       hiding (openURL)
 import           Text.Pandoc.Error                 (PandocError)
 import           Text.Parsec                       (ParseError)
 
@@ -90,7 +94,7 @@ import           Text.BlogLiterately.Highlight     (HsHighlight (HsColourInline)
                                                     getStylePrefs,
                                                     _HsColourInline)
 import           Text.BlogLiterately.Image         (uploadAllImages)
-import           Text.BlogLiterately.LaTeX         (wpTeXify)
+import           Text.BlogLiterately.LaTeX         (rawTeXify, wpTeXify)
 import           Text.BlogLiterately.Options
 import           Text.BlogLiterately.Options.Parse (readBLOptions)
 import           Text.BlogLiterately.Post          (findTitle, getPostURL)
@@ -164,6 +168,10 @@ runTransforms ts bl p = execStateT (mapM_ runTransform ts) (bl,p)
 -- These transforms are enabled by default in the standard
 -- @BlogLiterately@ executable.
 
+-- | Pass LaTeX (inline or display) through unchanged (if the @rawlatex@ flag is set).
+rawtexifyXF :: Transform
+rawtexifyXF = pureTransform (const rawTeXify) rawlatex'
+
 -- | Format embedded LaTeX for WordPress (if the @wplatex@ flag is set).
 wptexifyXF :: Transform
 wptexifyXF = pureTransform (const wpTeXify) wplatex'
@@ -476,6 +484,8 @@ loadProfile bl =
 --
 --   * 'titleXF': extract the title from a special title block
 --
+--   * 'rawtexifyXF': pass LaTeX through unchanged
+--
 --   * 'wptexifyXF': turn LaTeX into WordPress format if requested
 --
 --   * 'ghciXF': run and typeset ghci sessions if requested
@@ -505,6 +515,7 @@ standardTransforms =
     -- much, except highlightOptsXF should go before highlightXF.
   , passwordXF
   , titleXF
+  , rawtexifyXF
   , wptexifyXF
   , ghciXF
   , uploadImagesXF
@@ -522,16 +533,16 @@ standardTransforms =
 -- | Transform a complete input document string to an HTML output
 --   string, given a list of transformation passes.
 xformDoc :: BlogLiterately -> [Transform] -> String -> IO (Either PandocError (BlogLiterately, String))
-xformDoc bl xforms =
-        fixLineEndings
-    >>> parseFile parseOpts
-    >>> traverse
-      (  runTransforms xforms bl
-      >=> (\(bl', p) -> return $ (bl', writeHtml (writeOpts bl') p) )
-      >=> _2 (return . renderHtml)
-      )
+xformDoc bl xforms = runIO .
+    (     fixLineEndings
+      >>> T.pack
+      >>> parseFile parseOpts
+      >=> (liftIO . runTransforms xforms bl)
+      >=> (\(bl', p) -> (bl',) <$> writeHtml5String (writeOpts bl') p)
+      >=> _2 (return . T.unpack)
+    )
   where
-    parseFile :: ReaderOptions -> String -> Either PandocError Pandoc
+    parseFile :: ReaderOptions -> Text -> PandocIO Pandoc
     parseFile opts =
       case bl^.format of
         Just "rst"      -> readRST      opts
@@ -544,26 +555,37 @@ xformDoc bl xforms =
             _       -> readMarkdown opts
 
     parseOpts = def
-                { readerExtensions =
-                    case bl^.litHaskell of
-                      Just False -> readerExtensions def
-                      _          -> S.insert Ext_literate_haskell
-                                    (readerExtensions def)
-                , readerSmart      = True
-                }
+      { readerExtensions = pandocExtensions &
+          foldr (.) id
+            [ enableExtension Ext_tex_math_single_backslash
+            , case bl^.litHaskell of
+                Just False -> id
+                _          -> enableExtension Ext_literate_haskell
+            ]
+
+            -- Relevant extensions enabled by default in pandocExtensions:
+            -- (see https://hackage.haskell.org/package/pandoc-2.1.1/docs/src/Text-Pandoc-Extensions.html#pandocExtensions):
+
+            -- Ext_smart
+            -- Ext_yaml_metadata_block
+            -- Ext_tex_math_dollars
+            -- Ext_pandoc_title_block
+            -- Ext_citations
+      }
     writeOpts bl = def
-                   { writerReferenceLinks = True
-                   , writerTableOfContents = toc' bl
-                   , writerHTMLMathMethod =
-                     case math' bl of
-                       ""  -> PlainMath
-                       opt -> mathOption opt
-                   , writerTemplate       = Just blHtmlTemplate
-                   }
+      { writerReferenceLinks = True
+      , writerTableOfContents = toc' bl
+      , writerHTMLMathMethod =
+          case math' bl of
+            ""  -> PlainMath
+            opt -> mathOption opt
+      , writerTemplate       = Just blHtmlTemplate
+      }
+
     mathOption opt
       | opt `isPrefixOf` "latexmathml" ||
         opt `isPrefixOf` "asciimathml" = LaTeXMathML (mathUrlMaybe opt)
-      | opt `isPrefixOf` "mathml"      = MathML (mathUrlMaybe opt)
+      | opt `isPrefixOf` "mathml"      = MathML
       | opt `isPrefixOf` "mimetex"     =
           WebTeX (mathUrl "/cgi-bin/mimetex.cgi?" opt)
       | opt `isPrefixOf` "webtex"      = WebTeX (mathUrl webTeXURL opt)
